% PART 1: Initialize the code

scrsz = get(0,'ScreenSize'); % get the current screen resolution
figure(3);
if figure(1)
    clf(1,'reset');
end
set(figure(1),'Name','Inversion of Geoacoustic Parameters using Genetic Algorithms','NumberTitle','on','Position',[40 100 scrsz(3)/2.1 round(scrsz(4)/1.3)],'Tag','Plotting Figure');
start_button = uicontrol('String','Start','Position',[scrsz(3)/2.2-40 scrsz(4)/3 72 36],'Enable','on','callback','[push1]=pushbutton(push1)');
end_button = uicontrol('String','End','Position',[scrsz(3)/2.2-40 scrsz(4)/3-50 72 36],'Enable','on','callback','[push]=pushbutton(push)');
pause_button = uicontrol('String','Pause','Position',[scrsz(3)/2.2-40 scrsz(4)/3-100 72 36],'Style','checkbox','Enable','on','callback','[pausebutton]=pushbutton(pausebutton)');

% initialize; set boundaries, precision, etc:

%
%
drawnow
push1=1;

while push1==0              % Press "Start" to start the inversion.
    pause(0.1);
end

%presga      % get the inversion parameters, boundaris and discretizing.

pausebutton=0;           
push=0;

% PART 2: Set inversion options

% Initialize
filedata = readfile_initz('observed.pre',n);        % Import som info, so we can reproduse .pre files later

%nrec = str2double(filedata{2}{3,1}(5:15));  % Number of receivers
nPar = size(Field.length,2);                % number of parameters to estimate

[obs_data, filesize] = observed_data('observed.out',n);

ncross = 0;
nmutate = 0;
gen = 1;
push=0;
T0=100;

min_phen=[];    
avg=[];
min_gen=[];
BestObj = ones(20,1)*inf;
BestPhen = zeros(20, nPar);

% Delete old files not needed
name = 'model_*';
delete(name);




% PART 3:                     Start the inversion!!!

tic;    % Start the timer

%Initialize the first population:
NewChrom = initialize(Options.popsize,Options.lchrom,Options.nsubpop);


PhenOld = decode(NewChrom, Field);              % decode to population to their real values

write_file(PhenOld, filedata, model_matrix, nlayers, nParm, inv_parameters );   %Write pre files for every modelvector

% The call to the forward modell OSIRIS
call_Osiris(Options.popsize, filesize);
fprintf('\n\nProcessing...\n\n');
%%

data_inn = importdata(Options.popsize,n);         % Import all the generated frequncy responses

ObjValOld = objfunc2(obs_data, data_inn);

           
[BestObj, BestPhen] = storebest(PhenOld, BestPhen, ObjValOld, BestObj);     %Make sure to save the 20 best

% Calculate some statistics:
max_gen(gen) = max(ObjValOld);
[min_gen(gen), A] = min(ObjValOld);
avg(gen) = mean(ObjValOld);
min_phen(gen,:) = PhenOld(A,:);

OldChrom = NewChrom;

% Its time for some plotting:
figure(1);
% PLOTTS:    
    % Plot the parameter values of the best individual in each generation
for i = 1:nParm    
    subplot(2,nParm,i);
    idx = 1+nlayers*(i-1):nlayers*i;
    switch (inv_parameters(i,:))
        case ('ti')
            idx = idx(1:end-1);
    end    
    bar(PhenOld(A,idx));title('Current best individual');xlabel(['parameter: ', inv_parameters(i,:)]);
    for u=1:size(idx,2)
        text(u-0.36, min_phen(gen,idx(u))*0.9,['\color{white}' num2str(ceil(min_phen(gen,idx(u))),'%.0f')]);
    end
end
    % Plot the value of the objective function, best and average
    subplot(2,nParm,nParm+1:2*nParm); plot(min_gen,'--w.','LineWidth',2,'MarkerEdgeColor','r','MarkerFaceColor','g','MarkerSize',10);
    hold on; plot(avg,':w.','LineWidth',2,'MarkerEdgeColor','b','MarkerFaceColor','b','MarkerSize',10); hold off;
    axis([0 Options.maxgen 0 inf]);
    title(['Best = ', num2str(min_gen(gen)),'   Mean = ',num2str(avg(gen))],'Units','normalized');
    xlabel('Generation'); ylabel('f(x)');
    
    figure(3);  plotgeneration(PhenOld, nParm, nlayers, inv_parameters);
    
%%    
    
while gen < Options.maxgen        % Main iterative loop (feedback-loop)
        
    gen=gen+1;                    % increase the generation counter

    % Find the best ind. too be kept and later for reinsertion
    [ans,I] = sort(ObjValOld,1,'descend');
    
% CALCULATE EXPECTET NUMBER OF PARENTS FOR EACH INDIVIDUAL (CLACULATE FITNESS):
     p=prob(ObjValOld, min_gen(gen-1),GGAP, gen);        % Calculate with probability based on temperature
%      p = scale(ObjValOld);                               % Linear based fitness ranking  
%      p = fitsrank(ObjValOld, Options.popsize);           % Exponential based fitness ranking  


% CHOOSE PARENTS BASED ON FITNESS:
     parents = sel_roulette(p, GGAP);                    % Roulette Wheel Selection
%     parents = stochastic_selection(GGAP,p,1);          % Remainder stochastic selection 
%     parents = sus(p, GGAP);                            % Stochastic universal selection
    
% Shuffle the selected parents before crossover
    parents = randsample(parents,length(parents));
% 
% CROSSOVER:
%      [NewChrom,n] = crossover(OldChrom, Options.pcross, parents);                      % Single point         
%      NewChrom = mulpointxover2(OldChrom, Options.pcross, parents,Field.length);        % multiple point crossover
     NewChrom = xovermulti(OldChrom, Options.pcross, parents, 4);                       % multipoint crossover
    
% MUTATION:
    [NewChrom, nmutate] = mutation(NewChrom, Options.pmutate, nmutate);
    
% DECODING & EVALUATION (Again...):    
    PhenNew = decode(NewChrom, Field);
    
    % Update the alreay created pre-files with the new parameters
    o_write(PhenNew, model_matrix, nlayers, nParm, inv_parameters);

% call osiris and calculate
    call_Osiris(GGAP, filesize);
    fprintf('\nProcessing\n\n');
    
    data_inn_new = importdata(GGAP,n);
%    data_inn_new = process(data_inn_new, X, nrec);
    ObjValNew = objfunc2(obs_data, data_inn_new); 

        
% REINSERTION:
    OldChrom(I(1:GGAP),:) = NewChrom(:,:);
    ObjValOld(I(1:GGAP),:) = ObjValNew(:,:);
    PhenOld(I(1:GGAP),:) = PhenNew(:,:);
    data_inn(:,I(1:GGAP)) = data_inn_new(:,:);

    [BestObj, BestPhen] = storebest(PhenOld, BestPhen, ObjValOld, BestObj);
    
% STATISTICS:
    [min_gen(gen), A] = min(ObjValOld);
    avg(gen) = mean(ObjValOld);
    min_phen(gen,:) = PhenOld(A,:);

% PLOTTS:    
    % Plot the value of the objective function, best and average
    figure(1);
     for i = 1:nParm    
        subplot(2,nParm,i);
        idx = 1+nlayers*(i-1):nlayers*i;
        switch (inv_parameters(i,:))
            case ('ti')
                idx = idx(1:end-1);
        end
        bar(PhenOld(A,idx));title('Current best individual');xlabel('Layer');
        title(['parameter: ', inv_parameters(i,:)])
        for u=1:size(idx,2)
            text(u-0.36, min_phen(gen,idx(u))*0.9,['\color{white}' num2str(ceil(min_phen(gen,idx(u))),'%.0f')]);
        end
    end
    % Plot the value of the objective function, best and average
    subplot(2,nParm,nParm+1:2*nParm); semilogy(min_gen,'--w.','LineWidth',2,'MarkerEdgeColor','r','MarkerFaceColor','g','MarkerSize',10);
    hold on; semilogy(avg,':w.','LineWidth',2,'MarkerEdgeColor','b','MarkerFaceColor','b','MarkerSize',10); hold off;
    axis([0 Options.maxgen 0 inf]);
    title(['Best = ', num2str(min_gen(gen)),'   Mean = ',num2str(avg(gen))],'Units','normalized');
    xlabel('Generation'); ylabel('f(x)');    
    
     t=toc;
    tb=annotation('textbox',[0.6 0.3 0.15 0.1],'BackgroundColor','w','String',...
    ['Cpu-time: ', num2str(floor(t/60)),' m ',num2str(round(rem(t,60)))]);
    % Plot alle modeller i hver generajson og grupper etter parameter
    figure(3);  plotgeneration(PhenOld, nParm, nlayers, inv_parameters);
    %bar(PhenOld','group'); set(gca,'YTick',[0:100:3200]); grid on;
    
%     set(axes,'YGrid','on');
    grid on;
    drawnow;
    
    if push
        gentemp=gen;
        gen=Options.maxgen;
    elseif pausebutton 
        while pausebutton
            pause(0.1);
        end
    end
end
if push
    gen=gentemp;
end


pause(0.1);

fclose('all');

filename = savevar(model)
save(filename)
pause(0.1);

clear;

model = 'OSIRIS';