function [Phen] = decode(chrom,Field)
% Decode the individuals to their phenotypic values

% format Field:
%				[length;		(num)
%				 code;		(0=binary     | 1=gray)
%                bound;     (0=No         | 1=Yes
%				 lb;		(num)
%				 ub;		(num)
%
%			   where
%				length   - row vector containing the length of
%					each substring in Chrom. sum(len)
%					should equal the individual length.
%				code  - binary row vector indicating how each
%					substring is to be decoded.
%               bound    - Include Boundaries
%				lb,
%				ub    - Lower and upper bounds for each
%					variable. 

[Nind,Lind] = size(chrom);
Nvar = length(Field.length);

if sum(Field.length) ~= Lind,
	error('Data in Field.lenght must agree with chromosome length');
end

% If boundaries are includet calculate the step size
if Field.bound
    deltaj = (Field.ub-Field.lb)./(2.^Field.length -1);
end

endp = cumsum(Field.length);
startp = cumsum([1 Field.length]);
startp = startp(1:end-1);

% Decode chromosomes

Phen = zeros(Nind,Nvar);
for i=1:Nvar
    idx = startp(i):endp(i);
    temp = chrom(:,idx);
    Phen(:,i) = bin2int(temp,Field.code);       % Convert binary or gray to its decimal eqivalent
    if Field.bound                              % If boundaries are includet, calculate the correct decimal value
        Phen(:,i) = Field.lb(i) + deltaj(i).*Phen(:,i);
    end
end
        
