close all;
clear all;

hf = forward;

waitfor(hf);

load forwmodel;

hm = modell;

waitfor(hm);
load geomodel;

hl = limit;
waitfor(hl);
load bounds;

% If the number of bits is 0, this means that this variable is not to be
% estimated, and the upper and lower bounds for the variable is removed
ind = find(bits ~= 0);
bits = bits(ind);
upper_bounds = upper_bounds(ind);
lower_bounds = lower_bounds(ind);
order = order(ind); %Just a temporary help in the programming

model_matrix = [cumsum(ti') de' vp' vs'] ./ 1000;


nParm = size(inv_parameters,1);
nlayers = n - 1;

hg = genvar;
waitfor(hg);

load genvariables;

% Construction of the Field structure:
Field = struct('length',bits,...
    'code',1,...
    'bound',1,...
    'lb',lower_bounds,...
    'ub',upper_bounds);

%				[length;	(vector)
%				 code;		(0=binary     | 1=graycoded binary)
%                bound;     (0=No         | 1=Yes
%				 lb;		(num)
%				 ub;		(num)
%
%			   where
%				length   - row vector containing the length (bitprecision),
%                   of each substring in Chrom. sum(lenth)
%					should equal the individual length.
%				code  - binary row vector indicating how each
%					substring is to be decoded.
%               bound    - Include boundaries
%				lb,
%				ub    - Vector containing Lower and upper bounds for each
%					parameter to be estimated.

% Construction of the Options structure:
Options = struct('popsize',popsize,...
    'lchrom',sum(Field.length),...
    'maxgen',maxgen,...
    'pcross',pcross,...
    'pmutate',pmutate,...
    'nsubpop',1,...
    'gengap',gengap);


%Options.pmutate = (1/Options.lchrom);

%               popsize      (int)
%                   -  the size of the population, scalar
%
%               lchrom      (int)
%                   - the length of the chromosones, must equal the sum of
%                     the phenomtypes length, scalar
%
%               maxgen     (int)
%                   - number of generations too produce, scalar
%
%               pcross      ([0,1])
%                   - Crossover rate. Scalar between 0 and 1,
%
%               pmutate     ([0,1])
%                   - Mutation rate. Scalar between 0 and 1
%
%               nsubpop     (int)
%                   - Number of subpopulations to be produced, if nsubpop
%                     is NaN, 1 is assumed
%
%               gengap      ([0, 1])
%                   - fraction of the population to be replaced by
%                     selection
%


% Make sure the generation gap is a even number
if ceil((Options.popsize*Options.gengap))/2 ~= fix(ceil((Options.popsize*Options.gengap))/2)
    GGAP = ceil(Options.popsize*Options.gengap) -1 ;
else
    GGAP = ceil(Options.popsize*Options.gengap);
end



switch model
    case 'THM_caiti'
        %Making the simulated data
        if model_matrix(end,1) == Inf
            model_matrix(end,1) = 2.000;
        end
        
        hthm = forwparamTHM;
        waitfor(hthm);
        load paramTHM;
        
        writeTHM(model_matrix,min_freq, max_freq, freq_step, Vp_min, Vp_max, Vp_step, Root_Tol,MXiterations,NumberModes)
        
        eval(['!','mx1 mxinput.md nul mxresult.dat']);
        load mxresult.dat
        observed = mxresult;
        save observed observed
        
        % The inversion
        GA_thm
    case 'THM_haskell'
        
        if model_matrix(end,1) == Inf
            model_matrix(end,1) = 2.000;
        end
        
        hthmhas = forwparamTHMHaskell;
        waitfor(hthmhas);
        load paramTHMHaskell;
        
        [c_observed, k_observed] = haskell(de, vp, vs, ti, min_freq, max_freq, kstep, cmin, cmax, cstep);
    
        GA_thmHaskell
        
    case 'Raytrace'
        
        %We have to assume a known thickness to have a code which does not
        %run for a week. This is just to make sure I do not try to find it
        %by mistake...
        if inv_parameters(end,:) == 'ti'
            inv_parameters = inv_parameters(1:end-1,:);
            bits = bits(1:end-2);
            upper_bounds = upper_bounds(1:end-2);
            lower_bounds = lower_bounds(1:end-2);
            order = order(1:end-2);
            nParm = nParm-1;
        end
        
        wdepth = ti(1);
        ldepth =ti(2);
        
        c0 = vp(1); rho0=de(1);
        cp1=vp(2);  cs1=vs(2);  rho1=de(2); % cs1 is not used because the model assumes a fluid layer
        %But I am hoping to be able to include it later, and that is why
        %I've got it here.

        cp2=vp(end);  cs2=vs(end); rho2=de(end); 
        RUNP1=1;
        trans_loss = plane_ray_all(wdepth, ldepth, RUNP1, c0, rho0, cp1, cp2, cs1, cs2, rho1, rho2);
        
        observed_transloss = min(trans_loss,[],1);
        
        GA_ray;
        
    case 'OSIRIS'
               
        th = 0;
        th(2:n+1) = cumsum(ti);
        if th(end) == Inf
             th(end) = 1e30;
        end
        
        layer = 1:n;
        layer(end+1) =0;
        
        var1 = [.01 .01]; %Can't remember the names of the variables, so these are used while I figure it out
        var1(3:n+1) = .5;
        var2 = [0 0];
        var2(3:n+1)=.8;
        
        vp(2:end+1) = vp;
        vs(2:end+1) =vs;
        de(2:end+1) = de;
        
        model_matrix = [layer' th' vp' vs' var1' var2' de'];
        
        file = readfile_initz('initzfile.pre',4);
        write_observed(file,model_matrix);
        
        fprintf('Processing syntetic model \n');
        unix(['sgaosiris* observed.pre ']);
        fileobs = dir(['observed.out']);
        
        if fileobs.bytes < 10000
            while 1
            fprintf(['Waiting for OSIRIS... \n']);
            pause(.5);
            fileobs = dir(['observed.out']);
            if fileobs.bytes > 10000
                break
            end
            end
        end
        
        filesize = fileobs.bytes;
        
        GA_inversion;
end