function NewChrom = mulpointxover2(Chrom, Px, selected, leng)

% Multiple point crossover
% Each parameter has a possible crossover point.
% Only the bits inside the parameter that crossover is being performed is
% shifted.

Chrom = Chrom(selected,:);
[Nind,Lind] = size(Chrom);
NewChrom = zeros(Nind,Lind);


%if nargin < 5, Rs = 0; end

Xops = floor(Nind/2);
DoCross = rand(Xops,size(leng,2)) < Px;
odd = 1:2:Nind-1;
even = 2:2:Nind;

% Compute the effective length of each chromosome pair
Mask = ones(Nind/2, Lind);
Mask = cumsum(Mask')';

% Compute cross sites for each pair of individuals, according to their
% effective length and Px (two equal cross sites mean no crossover)
%xsites(:, 1) = Mask(:, Lind);


Shift = (ones(Xops,1)*leng-1).*DoCross;                       % genererer en matrise som bestr av lengden p hver paramter, de kolonnene som ikke skal krysses blir satt til 0
Xsites = ceil(Shift.*rand(Xops, length(leng))); % en matrise som viser krysningspunkter for hver parameter i kromosomet

[j1,j2] = find(not(Shift));
A=ones(Xops,1)*cumsum(leng);                         % en matrise som viser hvor en parameter stopper og neste begynner
B=Xsites(:,2:end);                              % Lagrer Krysspunktene utenom den de i frste paramter
Xsites(:,2:end)=A(:,1:end-1)+B;                 % generer en matrise med de reelle krysningspunktenen i befolkningen

for i = 1:size(j1,1)
    Xsites(j1(i),j2(i)) = sum(leng(1:j2(i)));
end

odd2=1:2:2*size(Xsites,2);                      % Oddetallene fra 1 til antall parametre*2
even2=2:2:2*size(Xsites,2);                     % Partall fra 1 til antall parametre*2

Xsites(:,odd2)=Xsites;                          % Genererer en matrise med de reelle krysnignspunkter
Xsites(:,even2)=A;                              % inkludert punkter der paramterene begynner/slutter

% Express cross sites in terms of a 0-1 mask
a = Xsites(:,ones(1,Lind)) < Mask;                  
for i=2:size(Xsites,2)
    a = a ==(Xsites(:,i*ones(1,Lind)) < Mask);
end
Mask = a;

% Perform crossover
NewChrom(odd,:) = (Chrom(odd,:).* Mask) + (Chrom(even,:).*(~Mask));
NewChrom(even,:) = (Chrom(odd,:).*(~Mask)) + (Chrom(even,:).*Mask);

% If the number of individuals is odd, the last individual cannot be mated
% but must be included in the new population
if rem(Nind,2),
  NewChrom(Nind,:)=Chrom(Nind,:);

end
