function plot_best(phen, nLayers, inv_par, scrsz, best)
% Plots the estimated parameters in phen, best is the best model in phen.
% scrsz is the sreensize
% inv_par is the parameters being inverted


set(figure(5),'Position',[1 1 scrsz(3)*3/4 round(scrsz(4)*3/4)])
clf(5);
yax = {};
x_lim = {};

tick = -nLayers:-1;
     
[N, npar] = size(phen);

n = size(inv_par,1);

for k = 1:n
    switch (inv_par(k,:))
        case ('Vp')
            label(k,:) = 'P-velocity (m/s)        ';
            yax{k} = 1400:100:3100;
            x_lim{k} = 'auto';
        case ('Vs')
            label(k,:) = 'S-velocity (m/s)        ';
            yax{k} = 400:100:1700;
            x_lim{k} = 'auto';
        case ('ap')
            label(k,:) = 'P-attenuation (db/lamda)';
            yax{k} = 0.2:0.1:1;
            x_lim{k} = [0.2 1];
        case ('as')
            label(k,:) = 'S-attenuation (db/lamda)';
            yax{k} = 0.2:0.1:1;
            x_lim{k} = [0.2 1];
        case ('de')
            label(k,:) = 'Density (kg/m3)         ';
             yax{k} = 0:100:2400;           
             x_lim{k} = 'auto';
        case ('ti')
            label(k,:) = 'Layer thickness (m)     ';
            yax{k} = 0:5:80;
            x_lim{k} = [20 80];
            
    end
end


for i=1:n
    for j=1:N
        idx = 1+nLayers*(i-1):nLayers*i;
        switch (inv_par(i,:))
            case ('ti')
                idx = idx(1:end-1);
        end            
        [x y] = stairs(phen(j,idx));
        x = -x + 0.5;
        x = [x; x(end)-1];
        y = [y; y(end)];
        figure(5); subplot(n,1,i); stairs(y,x,'LineWidth', 1); hold on;  
        xlabel(label(i,:)); ylabel('Layer');
    end
    if best>0
        [x y] = stairs(phen(best,idx));
        x = -x + 0.5;
        x = [x; x(end)-1];
        y = [y; y(end)];
        stairs(y,x,'r','LineWidth', 2);
    end
    set(gca,'XTick',yax{i}, 'YTick', -nLayers:-1, 'XMinorTick', 'on','XMinorGrid','on');
    xlim(x_lim{i});
    grid on;
    drawnow;
end
        