function plotgeneration(Phen, nParm, nlayers, inv_par)
% Plots every model in phen grouped after the parameters
% The result is a bar plot

for k = 1:nParm
    switch (inv_par(k,:))
        case ('Vp')
            label(k,:) = 'P-speed (m/s)           ';
            yax{k} = 0:200:3500;
            limy{k} = [1400 inf];
        case ('Vs')
            label(k,:) = 'S-speed (m/s)           ';
            yax{k} = 0:100:1800;     
            limy{k} = [300 inf];
        case ('ap')
            label(k,:) = 'P-attenuation (db/lamda)';
            yax{k} = 0:0.1:1;
            limy{k} = 'auto';
        case ('as')
            label(k,:) = 'S-attenuation (db/lamda)';
            yax{k} = 0:0.1:1;
            limy{k} = 'auto';
        case ('de')
            label(k,:) = 'Density (kg/m3)         ';
            yax{k} = 0:200:2500;
            limy{k} = [900 inf];
        case ('ti')
            label(k,:) = 'Layer thickness (m)     ';
            yax{k} = 0:5:100;
            limy{k} = [20 inf];   
    end
end

if nParm <= 2
    V = nParm;
    H = 1;
elseif nParm >= 3
    V = ceil(nParm/2);
    H = 2;
end    

for i = 1:nParm
    subplot(V,H,i);
    idx = 1+nlayers*(i-1):nlayers*i;
    switch (inv_par(i,:))
        case ('ti')
            idx = idx(1:end-1);
    end    
    h=bar(Phen(:,idx)','group');
    xlabel('Layer');
    ylim(limy{i});
    title(['parameter: ', label(i,:)])
    grid on;
    set(h,'EdgeColor', 'none','BarWidth', 1);
    colormap Jet;
    set(gca,'YTick',yax{i},'YMinorTick', 'on');     
    drawnow;
end
    
