function [ra rb gamma] = variables(c,alpha,betha)
% [ra rb gamma] = variables(c,alpha,betha)
%
%This function calculates ra, rb and gamma for a given vector c and the
%scalars or vectors alpha and betha. Alpha and Betha are also checked to
%make sure they are of same length and larger than 0.
%
%The formula for ra, rb and gamma are found on page 8 in the article.

if length(alpha) == 1
    cgreata = c > alpha;
    cgreatb = c > betha;
    ra = (sqrt((c./alpha).^2 - 1) .* cgreata) - (i .* sqrt(1 - (c./alpha).^2) .* ~cgreata);
    rb = (sqrt((c./betha).^2 - 1) .* cgreatb) - (i .* sqrt(1 - (c./betha).^2) .* ~cgreatb);
    gamma = 2 .* ( betha./c).^2;
elseif length(alpha) ~= length(betha) || length(alpha)<= 0 || length(betha) <=0
    print('Alpha and Betha must be of same lengths, and larger than 0');
    return;
else
    for m =1:length(alpha)
        cgreata = c > alpha(m);
        cgreatb = c > betha(m);
        ra(m,:) = (sqrt((c./alpha(m)).^2 - 1) .* cgreata) - (i .* sqrt(1 - (c./alpha(m)).^2) .* ~cgreata);
        rb(m,:) = (sqrt((c./betha(m)).^2 - 1) .* cgreatb) - (i .* sqrt(1 - (c./betha(m)).^2) .* ~cgreatb);
        gamma(m,:) = 2 .* ( betha(m)./c).^2;
    end
end